%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SIR
% 1) Inputs
%	x = matrix of covariates n * p (n samples stored in lines of dimension p)
%	y = output vector n * 1
%	nbtranches = number of slices
% 2) Outputs
%	matfin = matrix to diagonalize
%	passage = matrix of eigenvectors
% 	lambda = vector of eigenvalues

function [passage,l,matfin]=sir2(x,y,nbtranches)
	

	% step 1
	[n,p]=size(x);
	moy=mean(x,1);
	matcov=cov(x); 
	xcentre=x-ones(n,1)*moy;
	z = xcentre;

	% step 2
	[~,ind]=sort(y,'ascend');
	nbipt=floor(n/nbtranches);% nb of samples in the (nbtranches-1) first slices 
	nbidt=n-nbipt*(nbtranches-1);% nb of samples in the last slice
	
	for i=1:nbtranches-1
		nbh(i)=nbipt;
		proph(i)=nbipt/n;
	end;
	proph(nbtranches)=nbidt/n;
	nbh(nbtranches)=nbidt;

	% step 3
	moyh=zeros(nbtranches,p);
	for i=1:nbtranches
		for j=1:nbh(i)
			moyh(i,:)=moyh(i,:)+z(ind(j+(i-1)*nbipt),:);
		end;
		moyh(i,:)=moyh(i,:)/nbh(i);
    end
 	% step 4
	matfin=zeros(p,p);
	for i=1:nbtranches
		xx=moyh(i,:);
        yy=xx'*xx;
		matfin=matfin+proph(i)*yy;
	end;
    
	[v,lambda]=eig(matcov\matfin);
    l=diag(lambda);
    [l,pos]=sort(l,'descend');
	v=v(:,pos);
    passage=v;
	
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
