%% Expectation Maximization student SIR 
% 
% This funtion performs a robustified version of SIR, the algorithm iterates 
% until convergence. 
%    
% 
% function [B]=stsir(X,Y,nslices,dim,tr)
% 
% INPUT 
% X is the nxp matrix of predictors (n-number of samples, p-dimension)
% Y is the univariate response variable
% nslices is the number of desired slices
% dim is the dimension of the e.d.r. space to look for
% tr stopping criteria: relative increase in log-likelihood
% 
% OUTPUT
%
%B is a pxdim matrix containing a basis of the e.d.r. space 

function [B]=stsirI(X,Y,nslices,d,tr)

if nslices<=d
    disp('The number of slices must be greater than the d')
    B=0;
return;
end

if nargin<5 
   tr=10^-3; 
end
    
n=size(X,2);
p=size(X,1);




[~,ind]=sort(Y,'ascend');
nbipt=floor(n/nslices);% nb of samples in the (nbtranches-1) first slices 
nbidt=n-nbipt*(nslices-1);% nb of samples in the last slice
nbh=zeros(1,nslices);
for i=1:nslices
		nbh(i)=nbipt;
end
	nbh(nslices)=nbidt;

BIC=zeros(1,length(d));    
    
dim=d;

h = waitbar(0,'Student SIR Progress');    
    
%initialization 
ubar=ones(n,1);
%ubar=rand(n,1)/1000;
utilde=zeros(n,1);


    
scp=0;
jj=1;
B=zeros(p,dim);
deltaQ=9999;



while (deltaQ>tr) % stopping criteria: relative increase in log-likelihood
                            
    
f=zeros(nslices,1);
Mbar=mean(ubar);
Xbar_j=zeros(p,nslices);


for i=1:nslices
   
    for j=1:nbh(i)
      
        f(i)=f(i)+ubar(ind(j+(i-1)*nbipt));
        Xbar_j(:,i)=Xbar_j(:,i)+ubar(ind(j+(i-1)*nbipt))*X(:,ind(j+(i-1)*nbipt));
        
    end
    Xbar_j(:,i)=Xbar_j(:,i)/(f(i));
    f(i)=f(i)/n;
end 


W=zeros(nslices-1,nslices-1);
invW=zeros(nslices-1,nslices-1);

for i=1:nslices-1
 for j=1:nslices-1
 
     W(i,j)=f(i)*(i==j)-f(i)*f(j)/Mbar;
     invW(i,j)=1/f(i)*(i==j)+1/f(nslices);
 
 end
end

Xbar=zeros(p,1);
for i=1:n
   
    Xbar=Xbar+ubar(i)*X(:,i);
    
end
Xbar=Xbar/sum(ubar);

Sigma=zeros(p,p);
for i=1:n
   
    Sigma=Sigma+ubar(i)*(X(:,i)-Xbar)*(X(:,i)-Xbar)';
    
end
Sigma=Sigma*1/n;


M=zeros(nslices-1,p);
for i=1:nslices-1
    
   M(i,:)=f(i)*(Xbar_j(:,i)-Xbar)';
    
end



% Estimation of matrix Gamma
gammaMat=M'*invW*M;

[evec,eval]=eig((Sigma)^(-1)*gammaMat);
[~,pos]=sort(diag(eval),'descend');
evec=evec(:,pos);
scp=trace(orth(B)*orth(B)'*orth(evec(:,1:dim))*orth(evec(:,1:dim))')/dim;
B=evec(:,1:dim);

% Estimation of the matrix V

V=Sigma-gammaMat*B*(B'*gammaMat*B)^(-1)*B'*gammaMat;


% Estimation of C

C=invW*M*B*(B'*V*B)^(-1);

% Estimation of mu

estmu=Xbar-V*B*C'*(f(1:nslices-1)/mean(ubar)); 


inV=V^(-1);


   
alfa=inv_digamma(mean(utilde));

C(nslices,:)=0;
%%%%%%%%%%%%%%%


appo=0;
Q(jj)=0;
for i=1:nslices
    for j=1:nbh(i)
        zz=ind(j+(i-1)*nbipt);
    
    delta=1/2*(estmu+V*B*C(i,1:dim)'-X(:,zz))'*inV*(estmu+V*B*C(i,1:dim)'-X(:,zz));
    appo=appo+ubar(zz)*2*delta;
    
    Q(jj)=Q(jj)+log(1+delta);
    end
end

Q(jj)=n*(log(gamma(alfa+p/2))-1/2*log(det(V))-log(gamma(alfa)))-(alfa+p/2)*Q(jj);
%%%%%%%%%%%%%%%
if jj-1==0
deltaQ=999;
else
deltaQ=abs((Q(jj)-Q(jj-1))/Q(jj-1));
end

appo=0;
for i=1:nslices
    for j=1:nbh(i)
        zz=ind(j+(i-1)*nbipt);
    
    delta=1/2*(estmu+V*B*C(i,1:dim)'-X(:,zz))'*inV*(estmu+V*B*C(i,1:dim)'-X(:,zz));
    appo=appo+ubar(zz)*2*delta;
    
    ubar(zz)=(alfa+p/2)/(1+delta);
    utilde(zz)=digamma(alfa+p/2)-log(1+delta);
    
    end
end



jj=jj+1;
if(jj>1000)
   disp('maximum number of iteration exceeded');
   return
end


waitbar(tr/deltaQ,h,'Student SIR Progress');
end
close(h);
disp('------Student SIR------');
disp('iterations to converge:'),disp(jj);
 figure
 plot(Q);
 title('Log-likelihood');

